package org.apache.turbine.pipeline;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Turbine" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Turbine", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;

import org.apache.turbine.Pipeline;
import org.apache.turbine.RunData;
import org.apache.turbine.TurbineException;
import org.apache.turbine.Valve;
import org.apache.turbine.ValveContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * Valve that conditionally executes a set of valves (internally
 * contained in a <code>TurbinePipeline</code>).
 *
 * @author <a href="mailto:james@jamestaylor.org">James Taylor</a>
 * @author <a href="mailto:dlr@finemaltcoding.com">Daniel Rall</a>
 * @version $Id: ConditionalValve.java,v 1.4 2004/11/01 19:10:15 epugh Exp $
 */
public abstract class ConditionalValve
    extends BranchPointValve
{
    private static final Log log = LogFactory.getLog( ConditionalValve.class );
    private static final String pipelineClassName = 
        "org.apache.turbine.pipeline.TurbinePipeline";

    private String pipelineDescriptorPath;

    /**
     * Sets up our <code>TurbinePipeline</code> instance.
     */
    protected ConditionalValve()
    {
        pipelines = new Pipeline[] { new TurbinePipeline() };
    }


    public void setPipelineDescriptorPath(String pdp)
    {
        this.pipelineDescriptorPath = pdp;
    } 

    /**
     * Initializes all sub-pipelines.
     *
     * @see org.apache.turbine.Valve#initialize
     */
    public void initialize()
        throws Exception
    {
            // Turbine's standard Pipeline implementation can use
            // descriptors to define what Valves are attached to it.
        if (pipelineDescriptorPath != null) 
        {
            log.debug("ConditionalValve using descriptor path: " 
                      + pipelineDescriptorPath);
            Reader reader = new BufferedReader(new FileReader(pipelineDescriptorPath));
            XStream pipelineMapper = new XStream(new DomDriver()); // does not require XPP3 library
            pipelines[0] = (Pipeline) 
            (Pipeline) pipelineMapper.fromXML(reader);            
        }
        super.initialize();
    }


    /**
     * Adds the provided valve to the contained pipeline.
     */
    public void addValve( Valve valve )
    {
        pipelines[0].addValve( valve );
    }

    /**
     * @see org.apache.turbine.Valve#invoke
     */
    public void invoke( RunData data, ValveContext context )
        throws IOException, TurbineException
    {
        if ( shouldInvoke( data ) )
        {
            log.debug("Invoking conditional pipeline");
            pipelines[0].invoke( data );
        }

        context.invokeNext( data );
    }

    /**
     * Determine whether the contained valves should be
     * run. Subclasses should implement this method.
     *
     * @return Whether the contained valves should be run.
     */
    protected abstract boolean shouldInvoke( RunData data );
}
