package org.apache.turbine.pipeline;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Turbine" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Turbine", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.configuration.Configuration;
import org.apache.turbine.RunData;
import org.apache.turbine.TemplateContext;
import org.apache.turbine.Turbine;
import org.apache.turbine.TurbineException;
import org.apache.turbine.ValveContext;
import org.apache.turbine.modules.Module;

/**
 * Implements the RunData target portion of the "Turbine classic"
 * processing pipeline (from the Turbine 2.x series).
 *
 * @author <a href="mailto:jvanzyl@apache.org">Jason van Zyl</a>
 * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
 * @author <a href="mailto:dlr@finemaltcoding.com">Daniel Rall</a>
 * @author <a href="mailto:mikeh@apache.org">Mike Haberman</a>
 * @author <a href="mailto:james@jamestaylor.org">James Taylor</a>
 * @author <a href="mailto:mpoeschl@marmot.at">Martin Poeschl</a>
 * @version $Id: DefaultTargetValve.java,v 1.19 2004/11/01 20:08:41 epugh Exp $
 */
public class DefaultTargetValve
    extends AbstractValve
{
    private static final Log log = LogFactory.getLog(DefaultTargetValve.class);

    protected static final String DEFAULT_MODULE_TYPE = "screens";

    protected String targetModuleType = DEFAULT_MODULE_TYPE;

    /**
     * Creates a new instance with the <code>targetModuleType</code>
     * and <code>runner</code> members filled in from Turbine's
     * configuration.
     */
    public DefaultTargetValve()
    {

    }
    
    

    /* (non-Javadoc)
     * @see org.apache.turbine.Valve#initialize()
     */
    public void initialize() throws Exception {
        Configuration cfg = Turbine.getConfiguration();

        if (cfg != null)
        {
            // Get the module type
            targetModuleType =
                cfg.getString("pipeline.default.targetModuleType",
                              DEFAULT_MODULE_TYPE);
        }
    }
    /**
     * @see org.apache.turbine.Valve#invoke(RunData, ValveContext)
     */
    public void invoke(RunData data, ValveContext context)
        throws IOException, TurbineException
    {
        try
        {
            execute(data);
        }
        catch (Exception e)
        {
            throw new TurbineException(e);
        }

        // Pass control to the next Valve in the Pipeline
        context.invokeNext(data);
    }

    /**
     * Executes the <code>Valve</code>.
     *
     * @param data The run-time data.
     */
    protected void execute(RunData data)
        throws Exception
    {
        // Get the target that will determine the template / layout
        String target = data.getTarget();
        if ( target != null )
        {
            data.getResponse().setLocale(data.getLocale());
            data.getResponse().setContentType(data.getContentType());

            // Get the template context, already populated by any context
            // builders that were found for this target
            TemplateContext context = Module.getTemplateContext( data );
            context.put( "template", target );

            // Resolve the layout template for this target

            String layout = Turbine.getResolver()
                .getTemplate("layouts", target);

            // Use the renderer to render the layout,

            render( data, context, layout );
        }
        else
        {
            if ( log.isDebugEnabled() )
            {
                log.debug( "Target was null." );
            }
        }
    }

    protected void render( RunData data, TemplateContext context, String target  )
        throws Exception
    {
        if ( log.isDebugEnabled() )
        {
            log.debug( "Rendering target " + target );
        }

        Renderer r = new Renderer( data );

        // FIXME: Can we remove hardcoding here?

        context.put( "renderer", r );

        // Render the target

        String out = r.render( target );

        // Write the composed string to the response

        data.getOut().print( out );
    }
}

