package org.apache.turbine.pipeline;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Turbine" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Turbine", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import java.io.IOException;

import org.apache.turbine.Turbine;
import org.apache.turbine.RunData;
import org.apache.turbine.TurbineException;
import org.apache.turbine.ValveContext;
import org.apache.turbine.Resolver;
import org.apache.turbine.modules.Module;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Valve which determines the target for a request and runs the modules
 * associated with that target. This valve is intended to model a portion
 * of the 'Turbine classic pipeline' as defined by the behavior of turbine
 * release 2 
 *
 * @author <a href="mailto:jtaylor@apache.org">James Taylor</a>
 * @version $Id: RunModulesValve.java,v 1.8 2003/11/23 00:42:22 mpoeschl Exp $
 */
public class RunModulesValve 
    extends AbstractValve
{
    private static final Log log = LogFactory.getLog( RunModulesValve.class );
    
    /**
     * The default target module used for building up the
     * context.
     */
    private String targetModuleType;
    
    /**
     * Initialize this valve for use in the pipeline.
     */
    public void initialize()
        throws Exception
    {
        targetModuleType = Turbine.getConfiguration()
            .getString( "pipeline.default.targetModuleType" );
    }

    /**
     * @see org.apache.turbine.Valve#invoke(RunData, ValveContext)
     */
    public void invoke( RunData data, ValveContext context )
        throws IOException, TurbineException
    {
        try
        {
            runModules( targetModuleType, data );
        }
        catch ( Exception e )
        {
            throw new TurbineException( "Failure in RunModulesValve", e );
        }

        // Pass control to the next Valve in the Pipeline
        context.invokeNext( data );
    }
    
    /**
     * Runs the module chain associated with the target
     */
    public void runModules( String moduleType, RunData data )
        throws Exception
    {
        String target; 
        String oldTarget = "";
        
        Resolver resolver = Turbine.getResolver();
        
        Module module; 
        
        target = normalizeTarget( data.getTarget() );
        
        while( ! oldTarget.equals( target ) && target != null ) 
        {
            module = resolver.getModule( moduleType, target );

            if ( log.isDebugEnabled() )
            {
                log.debug( "Executing module " + module.getClass().getName() 
                           + " for target: " + target );
            }

            module.execute( data );

            // Modules might change the target:
            oldTarget = target;
            target = data.getTarget();
            if ( target != null ) 
            {
                target = normalizeTarget(target);
            }
        }
        
        // Set the target to the final normalized path, which will be used by
        // the resolver and the appropriate TemplateEngineService to locate the
        // template.
       
        if ( log.isDebugEnabled() )
        {
            log.debug( "Setting target to " + target );
        }
        
        data.setTarget( target );
    }
    
    /**
     * Normalizes the target path to the format used internally by the turbine 
     * classic style valves and resolvers. ',' will be translated to '/', 
     * multiple path seperators will be collapsed, and the result is guranteed
     * not to begin with a path seperator.
     */
    private String normalizeTarget( String target )
        throws Exception
    {
        if (target == null || target.length() == 0)
        {
            throw new Exception("RunModulesValve::normalizeTarget() target is null!");
        }

        StringBuffer buffer = new StringBuffer( target );

        char c;        

        int len = buffer.length();
        
        boolean lastWasDelim = true;
        
        for ( int j = 0; j < len; j++ )
        {
            c = buffer.charAt( j );
            
            if ( c == ',' || c == '/' )
            {
                if ( lastWasDelim )
                {
                    buffer.deleteCharAt( j );
                    
                    j--;
                    len--;
                }
                else
                {
                    buffer.setCharAt( j, '/' );
                }
                
                lastWasDelim = true;
            }
            else
            {   
                lastWasDelim = false;
            }
        }
        
        if (len == 0) 
        {
            new Exception( "Syntax error in target: '" + target + '\'');
        }
        
        return buffer.toString();
    }
}
